/*@
  theory uses Stack {
    inv: PREDICATE(Stack.Stack) =
      PRED(this: Stack.Stack):
        this.null = FALSE AND this.stack.null = FALSE AND
        0 <= this.n AND this.n <= this.stack.length;
  }
@*/
public class Stack
{
  private int[] stack;
  private int n;

  public Stack() /*@
    assignable this;
    ensures inv(VAR this) AND VAR n = 0;
  @*/
  {
    stack = new int[10];
    n = 0;
  }

  public boolean isEmpty() /*@
    requires inv(VAR this);
    ensures  inv(VAR this) AND (VALUE@NEXT = TRUE <=> VAR n = 0);
  @*/
  {
    return n == 0;
  }

  public int top() /*@
    requires inv(VAR this) AND VAR n > 0;
    ensures  inv(VAR this) AND (VALUE@NEXT = (VAR stack).value[VAR n-1]);
  @*/
  {
    return stack[n-1];
  }

  public int pop() /*@
    requires inv(VAR this) AND VAR n > 0;
    assignable n;
    ensures  inv(VAR this) AND (VALUE@NEXT = (VAR stack).value[OLD n-1])
        AND  VAR n = OLD n-1;
  @*/
  {
    int result = top();
    n = n-1;
    return result;
  }

  public void push(int v) /*@
    requires inv(VAR this) 
         AND (VAR n = (VAR stack).length => 
               2*(VAR stack).length+1 <= Base.MAX_INT);
    assignable stack, n;
    ensures inv(VAR this)
        AND VAR n = OLD n+1
        AND (VAR stack).value[OLD n] = VAR v;
  @*/
  {
    if (n == stack.length) 
      stack = resize();  
    stack[n] = v;
    n = n+1;
  }

  private int[] resize() /*@
    requires inv(VAR this) 
         AND 2*(VAR stack).length+1 <= Base.MAX_INT;
    ensures LET result = VALUE@NEXT IN
      result.null = FALSE AND 
      result.length > (OLD stack).length AND
      (FORALL(i: INT): 0 <= i AND i < VAR n =>
         result.value[i] = (OLD stack).value[i]);
  @*/
  {
    int[] stack0 = new int[2*stack.length+1];
    for (int i=0; i<n; i++) /*@
      invariant inv(VAR this) AND
        0 <= VAR i AND VAR i <= VAR n AND
        (VAR stack0).null = FALSE AND
        (VAR stack0).length > (OLD stack).length AND
        (FORALL(i: INT): 0 <= i AND i < VAR i =>
           (VAR stack0).value[i] = (OLD stack).value[i]);
      decreases VAR n - VAR i;
    @*/
      stack0[i] = stack[i];
    return stack0;
  }
}











