// ----------------------------------------------------------------------------
// ObjectValueSymbol.java
// An object value.
// $Id: ObjectVariableSymbol.java,v 1.8 2011/06/28 18:39:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.io.PrintWriter;

import fmrisc.ProgramExplorer.InOut.SourcePosition;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Logic.Type;
import fmrisc.ProgramExplorer.Syntax.Program.ObjectVariableDeclaration;
import fmrisc.ProgramExplorer.Syntax.Program.TypeExpression;

/**
 * 
 * An object value.
 *
 */
public final class ObjectVariableSymbol extends GlobalVariableSymbolBase 
{
  private final ObjectVariableDeclaration decl;
  
  /***************************************************************************
   * Construct object variable symbol.
   * @param csymbol the symbol of the class holding the declaration.
   * @param name the name of the symbol.
   * @param type the type of the symbol.
   * @param decl the declaration introducing the symbol.
   * @return the symbol or null if some problem occurred.
   **************************************************************************/
  public static ObjectVariableSymbol construct(ClassSymbol csymbol, 
    Identifier name, TypeExpression type, ObjectVariableDeclaration decl)
  {
    ObjectVariableSymbol symbol = 
      new ObjectVariableSymbol(csymbol, name, type, decl);
    Type type0 = csymbol.getPackage().getTypeTranslator().logicType(type);
    if (type0 == null) return null;
    symbol.setLogicType(type0);
    return symbol;
  }
  
  /***************************************************************************
   * Construct object variable symbol.
   * @param csymbol the symbol of the class holding the declaration.
   * @param name the name of the symbol.
   * @param type the type of the symbol.
   * @param declaration the declaration introducing the symbol.
   **************************************************************************/
  private ObjectVariableSymbol(ClassSymbol csymbol, 
    Identifier name, TypeExpression type, ObjectVariableDeclaration decl)
  {
    super(csymbol, name, type);
    this.decl = decl;
  }
  
  /***************************************************************************
   * Return declaration associated to symbol.
   * @return the declaration.
   **************************************************************************/
  public ObjectVariableDeclaration getDeclaration()
  {
    return decl;
  }
  
  /***************************************************************************
   * Print symbol on output stream.
   * @param out the output stream.
   **************************************************************************/
  public void print(PrintWriter out)
  {
    out.print("object variable ");
    getType().print(out);
    out.print(" ");
    getLongName().print(out);
    if (decl == null) return;
    SourcePosition pos = decl.getPosition();
    out.print(" (");
    out.print(pos.toString());
    out.print(")");
  }
}


