// ----------------------------------------------------------------------------
// ParameterSymbol.java
// The symbol for a method/constructor parameter.
// $Id: ParameterSymbol.java,v 1.10 2010/12/20 18:58:07 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.io.PrintWriter;

import fmrisc.ProgramExplorer.InOut.SourcePosition;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Program.Parameter;
import fmrisc.ProgramExplorer.Syntax.Program.TypeExpression;

/**
 * 
 * The symbol for a method/constructor parameter.
 *
 */
public final class ParameterSymbol extends LocalVariableSymbolBase
{  
  private ParamSymbol psymbol;
  private Parameter parameter;
  
  /***************************************************************************
   * Construct parameter symbol.
   * @param psymbol the parameterized entity holding the parameter.
   * @param parameter the parameter
   * @param type its type
   * @return the symbol or null if a problem occurred.
   **************************************************************************/
  public static ParameterSymbol construct(ParamSymbol psymbol, 
    Parameter parameter, TypeExpression type)
  {
    ParameterSymbol symbol = new ParameterSymbol(psymbol, parameter, type);
    return symbol;
  }
  
  /***************************************************************************
   * Construct parameter symbol.
   * @param psymbol the parameterized entity holding the parameter.
   * @param parameter the parameter
   * @param type its type
   **************************************************************************/
  private ParameterSymbol(ParamSymbol psymbol, Parameter parameter,
    TypeExpression type)
  {
    super(parameter.name, type);
    this.psymbol = psymbol;
    this.parameter = parameter;
    setLogicType(psymbol.getClassSymbol().getPackage().getTypeTranslator().
        logicType(type));
  }
  
  /****************************************************************************
   * Get the entity to which the variable belongs.
   * @return the entity.
   ***************************************************************************/
  public ParamSymbol getParamSymbol()
  {
    return psymbol;
  }
  
  /****************************************************************************
   * Get the declaration of this parameter.
   * @return the parameter declaration.
   ***************************************************************************/
  public Declaration getDeclaration()
  {
    return parameter;
  }
  
  /***************************************************************************
   * Print symbol on output stream.
   * @param out the output stream.
   **************************************************************************/
  public void print(PrintWriter out)
  {
    out.print("parameter ");
    getType().print(out);
    out.print(" ");
    getName().print(out);
    Declaration decl = psymbol.getDeclaration();
    if (decl == null) return;
    SourcePosition pos = decl.getPosition();
    out.print(" (");
    out.print(pos.toString());
    out.print(")");
  }
}

