// ----------------------------------------------------------------------------
// ThisSymbol.java
// The symbol for "this" object in a non-static context.
// $Id: ThisSymbol.java,v 1.8 2011/06/15 16:18:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.io.PrintWriter;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The symbol for "this" object in a non-static context.
 *
 */
public final class ThisSymbol extends LocalVariableSymbolBase
{  
  private ClassSymbol csymbol;
  
  /***************************************************************************
   * Construct "this" symbol.
   * @param csymbol the class of the "this" object.
   * @return the symbol or null if a problem occurred.
   **************************************************************************/
  public static ThisSymbol construct(ClassSymbol csymbol)
  {
    ThisSymbol symbol = new ThisSymbol(csymbol);
    return symbol;
  }
  
  /***************************************************************************
   * Construct "this" symbol.
   * @param csymbol the class of the "this" object.
   **************************************************************************/
  private ThisSymbol(ClassSymbol csymbol)
  {
    super(Identifier.construct("this"), csymbol.getType());
    this.csymbol = csymbol;
  }
  
  /***************************************************************************
   * Print symbol on output stream.
   * @param out the output stream.
   **************************************************************************/
  public void print(PrintWriter out)
  {
    out.print(getName());
  }
  
  /***************************************************************************
   * Get the class of this symbol.
   * @return the symbol of the class.
   **************************************************************************/
  public ClassSymbol getClassSymbol()
  {
    return csymbol;
  }
}


