// ----------------------------------------------------------------------------
// AtomicFormula.java
// An atomic formula.
// $Id: AtomicFormula.java,v 1.7 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An atomic formula.
 *
 */
public final class AtomicFormula extends TermApplication implements Formula
{
  public static final int precedence = 70; // see SpecLang.g
  
  /***************************************************************************
   * Construct application expression.
   * @param fun the function that is being applied.
   * @param args the arguments to which the function is applied.
   * @return the formula.
   **************************************************************************/
  public static AtomicFormula construct (Term fun, Term[] args)
  {
    return new AtomicFormula(fun, args);
  }
  
  /***************************************************************************
   * Construct application expression.
   * @param fun the function that is being applied.
   * @param args the arguments to which the function is applied.
   **************************************************************************/
  private AtomicFormula(Term fun, Term[] args)
  {
    super(fun, args);
  }
  
  /***************************************************************************
   * Get precedence value of the formula operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}


