// ----------------------------------------------------------------------------
// BinaryTerm.java
// Base class of binary terms.
// $Id: BinaryTerm.java,v 1.7 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * Base class of binary terms.
 *
 */
public abstract class BinaryTerm extends BinaryExpression implements Term
{
  private Type type = null; // the type annotation
  
  /****************************************************************************
   * Set type annotation of term.
   * @param type the type annotation.
   ***************************************************************************/
  public void setType(Type type)
  {
    this.type = type;
  }
  
  /****************************************************************************
   * Set type annotation of term.
   * @return the type annotation.
   ***************************************************************************/
  public Type getType()
  {
    return type;
  }
  
  /****************************************************************************
   * Construct binary term.
   * @param first the first term.
   * @param second the second term.
   ***************************************************************************/
  protected BinaryTerm(Term first, Term second)
  {
    super(first, second);
  }
  
  /***************************************************************************
   * Get first subterm.
   * @return the first subterm.
   **************************************************************************/
  public Term getFirst()
  {
    return (Term)first;
  }
  
  /***************************************************************************
   * Get second subterm.
   * @return the second subterm.
   **************************************************************************/
  public Term getSecond()
  {
    return (Term)second;
  }
  
  /***************************************************************************
   * Indicate whether operator is associative.
   * @return true if the operator is associative.
   **************************************************************************/
  public boolean isAssociative()
  {
    return false;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
