// ----------------------------------------------------------------------------
// LetExpression.java
// Base class of an expression with local definitions.
// $Id: LetExpression.java,v 1.6 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.ASTVisitorOld;
import fmrisc.ProgramExplorer.Syntax.ASTVisitor;

/**
 * 
 * Base class of an expression with local definitions.
 *
 */
public abstract class LetExpression extends ExpressionBase
{
  public static final int precedence = 10; // see SpecLang.g
  
  public final static String letKeyword = "LET";
  public final static String inKeyword = "IN";
  
  protected final Definition[] definitions;
  protected final Expression body;
  
  /***************************************************************************
   * Construct expression with local definitions.
   * @param d the definitions.
   * @param b the body of the expression.
   **************************************************************************/
  protected LetExpression(Definition[] d, Expression b)
  {
    definitions = d;
    body = b;
  }
  
  /***************************************************************************
   * Get definitions.
   * @return the definitions.
   **************************************************************************/
  public Definition[] getDefinitions()
  {
    return definitions;
  }
  
  /***************************************************************************
   * Get body.
   * @return the body.
   **************************************************************************/
  public Expression getBody()
  {
    return body;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    accept(definitions, visitor);
    body.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    accept(definitions, visitor);
    body.accept(visitor);
  }
  
  /***************************************************************************
   * Get precedence value of the formula operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
}
