// ----------------------------------------------------------------------------
// ReadsOnlyFormula.java
// A formula that states that no program variable may change.
// $Id: ReadsOnlyFormula.java,v 1.6 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.ASTVisitor;

/**
 * 
 * A formula that states that no program variable may change.
 *
 */
public class ReadsOnlyFormula extends WritesOnlyFormula
{  
  public static String keyword = "READSONLY";
  
  /****************************************************************************
   * Construct "readsonly" formula.
   * @return the new formula
   ***************************************************************************/
  public static ReadsOnlyFormula construct()
  {
    return new ReadsOnlyFormula();
  }
   
  /***************************************************************************
   * Get string representation of keyword.
   * @return string representation of the keyword.
   **************************************************************************/
  public String getKeyword()
  {
    return keyword;
  }
  
  /****************************************************************************
   * Construct "readsonly" formula.
   ***************************************************************************/
  protected ReadsOnlyFormula()
  {
    super(null);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}

