// ----------------------------------------------------------------------------
// RecordTerm.java
// A record of named values.
// $Id: RecordTerm.java,v 1.7 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * A record of named values.
 *
 */
public final class RecordTerm extends TermBase
{
  public final static String beginToken = "(#";
  public final static String endToken = "#)";
  
  private final ValuedIdentifier[] base;
  
  /****************************************************************************
   * Construct record term.
   * @param base the base terms.
   * @return the term.
   ***************************************************************************/
  public static RecordTerm construct(ValuedIdentifier[] base)
  {
    return new RecordTerm(base);
  }
  
  /****************************************************************************
   * Construct record term.
   * @param base the base terms.
   ***************************************************************************/
  private RecordTerm(ValuedIdentifier[] base)
  {
    this.base = base;
  }
  
  /****************************************************************************
   * Get base terms.
   * @return the base terms.
   ***************************************************************************/
  public ValuedIdentifier[] getBase()
  {
    return base;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    accept(base, visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    accept(base, visitor);
  }
}
