// ----------------------------------------------------------------------------
// The type of state values.
// $Id: StateType.java,v 1.6 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the types of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.ASTVisitorOld;
import fmrisc.ProgramExplorer.Syntax.ASTVisitor;

/**
 * 
 * The type of state values.
 *
 */
public final class StateType extends TypeBase
{ 
  public final static String stateKeyword = "STATE";
  
  // the return type of a corresponding state (may be null indicating "void")
  private final Type returnType;
  
  /****************************************************************************
   * Construct a state type.
   * @param ret the return type of a corresponding state (may be null 
   *            which indicates "void").
   * @return the type.
   ***************************************************************************/
  public static StateType construct(Type ret)
  {
    return new StateType(ret);
  }
  
  /****************************************************************************
   * Construct a state type.
   * @param ret the return type of a corresponding state (may be null 
   *        which indicates "void")
   ***************************************************************************/
  private StateType(Type ret)
  {
    returnType = ret;
  }
  
  /****************************************************************************
   * Get return type.
   * @return the return type (may be null which indicates "void")
   ***************************************************************************/
  public Type getReturn()
  {
    return returnType;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    if (returnType != null) returnType.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    if (returnType != null) returnType.accept(visitor);
  }
}


