// ----------------------------------------------------------------------------
// ValueDeclaration.java
// The declaration of a value.
// $Id: ValueDeclarationClass.java,v 1.4 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Semantics.*;

/**
 * 
 * The declaration of a value.
 *
 */
public class ValueDeclarationClass extends DeclarationBase 
  implements ValueDeclaration
{
  protected final Type type;
  
  /****************************************************************************
   * Create new declaration of a constant denoting a value.
   * @param name the name of the constant.
   * @param type its type
   * @return the declaration
   ***************************************************************************/
  public static ValueDeclarationClass construct(Identifier name, Type type)
  {
    return new ValueDeclarationClass(name, type);
  }
    
  /****************************************************************************
   * Create new declaration of a constant denoting a value.
   * @param name the name of the constant.
   * @param type its type
   * @return the declaration
   ***************************************************************************/
  protected ValueDeclarationClass(Identifier name, Type type)
  {
    super(name);
    this.type = type;
  }
  
  /***************************************************************************
   * Get type of the declaration.
   * @return the type (may be null).
   **************************************************************************/
  public final Type getType()
  {
    return type;
  }
  
  /***************************************************************************
   * Get symbol introduced by declaration.
   * @return the symbol.
   **************************************************************************/
  public final ValueSymbol getSymbol()
  {
    return (ValueSymbol)name.getSymbol();
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    type.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    type.accept(visitor);
  }
}
