// ----------------------------------------------------------------------------
// ValuedIdentifier.java
// An identifier with a value.
// $Id: ValuedIdentifier.java,v 1.5 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An identifier with a value.
 * 
 */
public final class ValuedIdentifier extends ASTLogicBase
{  
  private final Identifier name;
  private final Term value;
  
  /****************************************************************************
   * Construct an identifier with a value.
   * @param name the identifier.
   * @param value its value.
   * @return the valued identifier.
   ***************************************************************************/
  public static ValuedIdentifier construct(Identifier name, Term value)
  {
    return new ValuedIdentifier(name, value);
  }
  
  /****************************************************************************
   * Construct a variable with a value.
   * @param var the variable.
   * @param value its value.
   ***************************************************************************/
  private ValuedIdentifier(Identifier name, Term value)
  {
    this.name = name;
    this.value = value;
  }
  
  /***************************************************************************
   * Get identifier.
   * @return the identifier.
   **************************************************************************/
  public Identifier getIdentifier()
  {
    return name;
  }
  
  /***************************************************************************
   * Get value.
   * @return the value.
   **************************************************************************/
  public Term getValue()
  {
    return value;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    name.accept(visitor);
    value.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
    value.accept(visitor);
  }
}

