// ----------------------------------------------------------------------------
// WritesOnlyFormula.java
// A formula that states that only certain program variables may change.
// $Id: WritesOnlyFormula.java,v 1.8 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * A formula that states that only certain program variables may change.
 *
 */
public class WritesOnlyFormula extends FormulaBase
{
  public static String keyword = "WRITESONLY";
  private Name[] variables;
  
  /****************************************************************************
   * Construct "writesonly" formula.
   * @param variables the names of the variables that may change (null, if none)
   * @return the new formula
   ***************************************************************************/
  public static WritesOnlyFormula construct(Name[] variables)
  {
    return new WritesOnlyFormula(variables);
  }
   
  /****************************************************************************
   * Construct "writesonly" formula.
   * @param variables the names of the variables that may change (null, if none)
   ***************************************************************************/
  protected WritesOnlyFormula(Name[] variables)
  {
    this.variables = variables;
  }
  
  /****************************************************************************
   * Get names of variables that are allowed to change.
   * @return the names (null, if none)
   ***************************************************************************/
  public Name[] getVariables()
  {
    return variables;
  }
  
  /***************************************************************************
   * Get string representation of keyword.
   * @return string representation of the keyword.
   **************************************************************************/
  public String getKeyword()
  {
    return keyword;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    if (variables != null) accept(variables, visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    if (variables != null) accept(variables, visitor);
  }
}
