// ----------------------------------------------------------------------------
// CallStatement.java
// A method call statement.
// $Id: CallStatement.java,v 1.6 2011/09/30 15:47:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Semantics.*;

/**
 * An method call statement.
 */
public abstract class CallStatement extends StatementBase
{
  public final Name method;
  public final ValueExpression[] args;
  private ParamSymbol msymbol;
  
  /****************************************************************************
   * Construct a call statement.
   * @param name the method name
   * @param args the method arguments
   ***************************************************************************/
  protected CallStatement(Name method, ValueExpression[] args)
  {
    this.method = method;
    this.args = args;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
    method.accept(visitor);
    accept(args, visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    method.accept(visitor);
    accept(args, visitor);
  }
  
  /***************************************************************************
   * Set symbol of called method.
   * @param msymbol the method symbol.
   **************************************************************************/
  public void setMethodSymbol(ParamSymbol msymbol)
  {
    this.msymbol = msymbol;
  }
  
  /***************************************************************************
   * Get symbol of called method.
   * @return the method symbol.
   **************************************************************************/
  public ParamSymbol getMethodSymbol()
  {
    return msymbol;
  }
  
  /****************************************************************************
   * Get type of statement.
   * @return a string identifying the type of statement.
   ***************************************************************************/
  public String getKind()
  {
    return "procedure call";
  }
}
