// ----------------------------------------------------------------------------
// CompilationUnitBase.java
// Base class of a compilation unit
// $Id: CompilationUnitBase.java,v 1.6 2011/09/30 15:47:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * Base class of a compilation unit.
 */
public abstract class CompilationUnitBase 
extends DeclarationBase implements CompilationUnit
{
  public final Name pname;             // package name, may be null
  public final Import[] imports;       // the import statements
  
  /****************************************************************************
   * Create compilation unit.
   * @param pname the name of the unit's package (may be null).
   * @param imports the import statements.
   * @param name the name of the unit.
   ***************************************************************************/
  protected CompilationUnitBase(Name pname, Import[] imports, Identifier name)
  {
    super(name);
    this.imports = imports;
    this.pname = pname;
  }
  
  /****************************************************************************
   * Get package name of unit
   * @return the unit's package name (may be null)
   ***************************************************************************/
  public Name getPackage()
  {
    return pname;
  }

  /****************************************************************************
   * Get import statements of unit
   * @return the unit's import statements
   ***************************************************************************/
  public Import[] getImports()
  {
    return imports;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
    accept(imports, visitor);
    if (pname != null) pname.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    accept(imports, visitor);
    if (pname != null) pname.accept(visitor);
  }
}
