// ----------------------------------------------------------------------------
// ConstructorDeclaration.java
// A constructor declaration.
// $Id: ConstructorDeclaration.java,v 1.4 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import java.util.*;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * A constructor declaration.
 */
public final class ConstructorDeclaration extends ParamDeclarationBase
{  
  /****************************************************************************
   * Construction of a constructor declaration.
   * @param name the name of the constructor
   * @param params the constructor parameters
   * @param body the constructor body
   * @return the declaratin.
   ***************************************************************************/
  public static ConstructorDeclaration construct(Identifier name,
    Vector<Parameter> params, Statement body)
  {
    return construct(name, params.toArray(new Parameter[params.size()]),
        body);
  }
  
  /****************************************************************************
   * Construction of a constructor declaration.
   * @param name the name of the constructor
   * @param params the constructor parameters
   * @param body the constructor body
   * @return the declaration.
   ***************************************************************************/
  public static ConstructorDeclaration construct(Identifier name,
    Parameter[] params, Statement body)
  {
    return new ConstructorDeclaration(name, params, body);
  }
  
  /****************************************************************************
   * Construction of a constructor declaration.
   * @param name the name of the constructor
   * @param params the constructor parameters
   * @param body the constructor body
   ***************************************************************************/
  private ConstructorDeclaration(Identifier name,
    Parameter[] params, Statement body)
  {
    super(name, params, body);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
