// ----------------------------------------------------------------------------
// VariableStatement.java
// A local variable declaration
// $Id: VariableStatement.java,v 1.10 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Judgements.StatJudgement;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * A local variable declaration
 */
public final class VariableStatement 
extends VariableDeclarationBase implements Statement
{
  public final ValueExpression exp; // may be null
  private StatJudgement judgement;
  
  /****************************************************************************
   * Construct a local variable declaration
   * @param type the type of the variable
   * @param var the name of the variable
   * @param exp the initialization expression (may be null)
   * @return the statement.
   ***************************************************************************/
  public static VariableStatement construct
    (TypeExpression type, Identifier var, ValueExpression exp)
  {
    return new VariableStatement(type, var, exp);
  }
  
  /****************************************************************************
   * Construct a local variable declaration
   * @param type the type of the variable
   * @param var the name of the variable
   * @param exp the initialization expression (may be null)
   ***************************************************************************/
  private VariableStatement
    (TypeExpression type, Identifier var, ValueExpression exp)
  {
    super(var, type);
    this.exp = exp;
  }
  
  /****************************************************************************
   * Set command judgement.
   * @param judgement the judgement.
   ***************************************************************************/
  public void setJudgement(StatJudgement judgement)
  {
    this.judgement = judgement;
  }
  
  /****************************************************************************
   * Get command judgement.
   * @return the judgement
   ***************************************************************************/
  public StatJudgement getJudgement()
  {
    return judgement;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    if (exp != null) exp.accept(visitor);
    if (judgement != null) judgement.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    if (exp != null) exp.accept(visitor);
    if (judgement != null) judgement.accept(visitor);
  }
  
  /****************************************************************************
   * Get type of statement.
   * @return a string identifying the type of statement.
   ***************************************************************************/
  public String getKind()
  {
    return "declaration";
  }
}
