// ----------------------------------------------------------------------------
// EnvironmentC.java
// The "environment" command.
// $Id: EnvironmentC.java,v 1.8 2012/10/15 15:16:47 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import fmrisc.ProofNavigator.*;
import fmrisc.ProofNavigator.Proving.*;
import fmrisc.ProofNavigator.Semantics.*;

import java.io.PrintWriter;
import java.util.*;

/**
 * 
 * Command "environment": show environment (global or of current state).
 *
 */
public final class EnvironmentC extends CommandBase
{
  public final static String Name = "environment";
  
  /***************************************************************************
   * Create an "environment" command
   **************************************************************************/
  public EnvironmentC()
  {
    super(Name);
  }
  
  /****************************************************************************
   * Process command "environment": show environment (global or of 
   * current state).
   ***************************************************************************/
  public void process()
  { 
    PrintWriter out = State.getSystemOut();
    
    Environment env;
    Proof proof = State.getProof();
    if (proof == null)
     env = State.getEnvironment();
    else
     env = proof.getCurrent().getEnvironment();
    
    Iterator<Symbol> iter = env.getTypeSymbols().iterator();
    if (iter.hasNext())
    {
      out.print("Types: ");
      while (iter.hasNext())
      {
        TypeSymbol symbol = (TypeSymbol)iter.next();
        symbol.getIdentifier().print(out);
        if (iter.hasNext()) out.print(", ");
      }
      out.println(".");
    }

    iter = env.getValueSymbols().iterator();
    if (iter.hasNext())
    {
      out.print("Values: ");
      while (iter.hasNext())
      {
        ValueSymbol symbol = (ValueSymbol)iter.next();
        symbol.getIdentifier().print(out);
        out.print(":");
        symbol.getType().print(out);
        if (iter.hasNext()) out.print(", ");
      }
      out.println(".");
    }
    iter = env.getFormulaSymbols().iterator();
    if (iter.hasNext())
    {
      out.print("Formulas: ");
      while (iter.hasNext())
      {
        FormulaSymbol symbol = (FormulaSymbol)iter.next();
        symbol.getIdentifier().print(out);
        if (iter.hasNext()) out.print(", ");
      }
      out.println(".");
    }
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
  }
}