// ----------------------------------------------------------------------------
// FormulaTable.java
// Symbol table for formulas.
// $Id: FormulaTable.java,v 1.8 2017/07/04 08:04:35 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Semantics;

import fmrisc.ProofNavigator.Syntax.*;

/**
 * 
 * Symbol table for formulas.
 *
 */
public final class FormulaTable extends SymbolTable
{
  /****************************************************************************
   * creates a clone of the symbol table, the clone is a shallow copy
   * (the symbols in the table are not cloned)
   * @return the clone.
   ***************************************************************************/
  public FormulaTable cloneFormulaTable()
  {
    return (FormulaTable)(super.cloneTable(new FormulaTable()));
  }
  
  /****************************************************************************
   * put formula in formula table
   * @param key the name of the formula
   * @param formula the formula associated to the name
   * @param axiom true iff formula is axiom
   * @param env the environment in which the formula was declared
   * @return constructed symbol (null, if an error occured)
   ***************************************************************************/
  public FormulaSymbol put(FormulaDeclIdentifier key, Expression formula,
    boolean axiom, Environment env)
  {
    FormulaSymbol symbol = key.getSymbol();
    symbol.setFormula(formula);
    symbol.setEnvironment(env.cloneGlobal());
    if (axiom) symbol.setAxiom();
    super.put(key, symbol); 
    return symbol;
  }
  
  /****************************************************************************
   * returns symbol associated to name (null, if none)
   * @param name the name 
   * @return the symbol associated to the name (may be null)
   ***************************************************************************/
  public FormulaSymbol getSymbol(Identifier name)
  {
    return (FormulaSymbol)(get(name));
  }
  
  /****************************************************************************
   * returns formula associated to key
   * @param key the name of the formula
   * @return the expression associated to the formula
   ***************************************************************************/
  public Expression getFormula(Identifier key)
  {
    FormulaSymbol symbol = getSymbol(key);
    return symbol.getFormula();
  }
}

