// ----------------------------------------------------------------------------
// TypeExpression.java
// Pair of type and formula.
// $Id: TypeExpression.java,v 1.8 2017/07/04 08:04:35 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// -------------------------------------------------------------------------
package fmrisc.ProofNavigator.Semantics;

import fmrisc.ProofNavigator.Syntax.*;

/**
 * 
 * Pair of type and exp.
 *
 */
public final class TypeExpression
{
  private Type type;
  private Expression exp;
  
  /***************************************************************************
   * Construct pair of type and exp
   * @param type the type
   * @param exp the exp
   **************************************************************************/
  public TypeExpression(Type type, Expression exp)
  {
    this.type = type;
    this.exp = exp;
  }
  
  /***************************************************************************
   * get exp 
   * @return the exp
   **************************************************************************/
  public Expression getExpression()
  {
    return exp;
  }

  /***************************************************************************
   * set exp 
   * @param exp the expression
   **************************************************************************/
  public void setExpression(Expression exp)
  {
    this.exp = exp;
  }

  /***************************************************************************
   * get type
   * @return the type
   **************************************************************************/
  public Type getType()
  {
    return type;
  }

  /***************************************************************************
   * set type
   * @param type the type
   **************************************************************************/
  public void setType(Type type)
  {
    this.type = type;
  }
  
  /***************************************************************************
   * create instantiated copy of type expression
   * @return the copy.
   **************************************************************************/
  public TypeExpression instantiate()
  {
    Expression exp0 = null;
    if (exp != null) exp0 = exp.instantiate();
    return new TypeExpression(type.instantiateType(), exp0);
  }
}
