// ----------------------------------------------------------------------------
// Expression.java
// Interface to expressions (terms and formulas).
// $Id: Expression.java,v 1.6 2008/10/13 09:32:01 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Syntax;

import java.io.PrintWriter;

/**
 * 
 * Interface to expressions (terms and formulas).
 *
 */
public interface Expression extends AST
{
  /****************************************************************************
   * Get Priority of expression (higher values mean less binding power).
   * @return the expression priority.
   ***************************************************************************/
  public int getPriority();
    
  /****************************************************************************
   * print expression, use parentheses if priority is greater than treshold
   * @param treshold priority level of enclosing expression
   * @param out writer to print expression on
   ***************************************************************************/
  public void printPriority(int treshold, PrintWriter out);
  
  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Expression instantiate();
}
