// ----------------------------------------------------------------------------
// SelectionTerm.java
// Handling of component access by index selections.
// $Id: SelectionTerm.java,v 1.12 2008/10/13 09:32:00 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Syntax;

import java.io.*;

/**
 * Handling of component access by index selections.
 */
public final class SelectionTerm extends ExpressionBase
{
  private Expression base1; // the compound object
  private Selector   base2; // the selector
  
  /****************************************************************************
   * construct component selection from base1 and base2
   * @param base1 the compound object
   * @param base2 the selector
   ***************************************************************************/
  public SelectionTerm(Expression base1, Selector base2)
  {
    this.base1 = base1;
    this.base2 = base2;
  }
  
  /***************************************************************************
   * get binding level for printing (lower numbers mean greater binding power)
   * @return the priority level
   **************************************************************************/
  public int getPriority()
  {
    return 5;
  }
  
  /***************************************************************************
   * get base expression
   * @return the base expression
   **************************************************************************/
  public Expression getBase()
  {
    return base1;
  }
  
  /***************************************************************************
   * get selector
   * @return the selector expression
   **************************************************************************/
  public Selector getSelector()
  {
    return base2;
  }

  /***************************************************************************
   * return copy of AST with value references instantiated by the substitutions
   * set in the corresponding declaration identifiers
   * @return the instantiated copy
   **************************************************************************/
  public Expression instantiate()
  {
    return Construct.selectionTerm(
      getBase().instantiate(), getSelector().instantiate());
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    base1.printPriority(getPriority(), out);
    base2.print(out);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be nul
   **************************************************************************/
  public AST accept(ASTVisitor visitor) 
  { 
    AST result = visitor.visit(this);
    if (result != null) return result;
    AST base10 = base1.accept(visitor);
    AST base20 = base2.accept(visitor);
    if (!visitor.isCloned()) return null;
    return Construct.selectionTerm((Expression)base10, (Selector)base20);
  }
}